/***************************************************************************
                          kspelledit.cpp  -  description
                             -------------------
    begin                : Tue Jan 21 2003
    copyright            : (C) 2003 by Zach Hobbs
    email                : zach@firstwavedesign.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kspelledit.h"

KSpellEdit::KSpellEdit(QWidget *parent, const char *name ) : QWidget(parent,name) {

  
  QVBoxLayout *layout = new QVBoxLayout( this );
  QHBoxLayout *spellNameRow = new QHBoxLayout(  );
  QHBoxLayout *spellPathRow = new QHBoxLayout(  );
  QHBoxLayout *buttonLayout = new QHBoxLayout(  );
  
  layout->setMargin( 20 );
  layout->setSpacing( 20 );

  //Add HBoxes
  layout->addLayout( spellNameRow );
  layout->addLayout( spellPathRow );
  layout->setAlignment(Qt::AlignCenter);
    
  //Spell Name
  QLabel *spellNameLabel = new QLabel( "Spell Name: ", this );
  spellName = new QLabel(this);
  
  spellNameRow->addWidget( spellNameLabel );
  spellNameRow->addWidget( spellName );

  //Spell Path
  QLabel *spellPathLabel = new QLabel( "Spell Path: ", this );
  spellPath = new QLabel(this);

  spellPathRow->addWidget( spellPathLabel );
  spellPathRow->addWidget( spellPath );

  // ComboBox to select file
  fileSelect = new QComboBox( false, this );
  fileSelect->insertItem( " - Select File - ", 0 );
  
  layout->addWidget( fileSelect );

  // File Edit
  editBox = new KFileEdit( this );
  
  QFont fixed = KGlobalSettings::fixedFont();
  fixed.setPointSize(10);
  editBox->setFont( fixed );
  editBox->setMinimumHeight( 300 );
  layout->addWidget(editBox);
  layout->addLayout( buttonLayout );

  // Make save button
  saveButton = new QPushButton("Save File", this );
  saveButton->setMaximumWidth( 100 );
  saveButton->setDisabled( true ); 
  buttonLayout->addWidget(saveButton);

  //Connections:

  //Files Selection change
  QObject::connect( fileSelect, SIGNAL( activated( int ) ),
		    this, SLOT( changeFile( int ) ) );

  //Save Button to save
  QObject::connect( saveButton, SIGNAL( clicked() ),
		    editBox, SLOT( saveFile() ) );

  //Check to see if editbox modified
  QObject::connect( editBox, SIGNAL( modificationChanged( bool )),
		    this, SLOT( changeSaveButton( bool ) ) );
}
KSpellEdit::~KSpellEdit(){
}

void KSpellEdit::updateFileList( QListViewItem *spell ) {

  if (spell->depth() == 2) {
    QDir spellDir( spell->text(4) );
    
    if (!spellDir.exists())
      return;
    
    spellName->setText( spell->text(0) );
    spellPath->setText( spell->text(4) );
    editBox->clear();

    spellDir.setFilter( QDir::Files );
    const QFileInfoList *fileList = spellDir.entryInfoList();

    QFileInfoListIterator it( *fileList );
    QFileInfo* fi;
    fileSelect->clear();
    fileSelect->insertItem( " - Select File - ", 0 );
      
    while( (fi = it.current()) ) {
      fileSelect->insertItem(fi->fileName() );
      ++it;
    }
  }
}
void KSpellEdit::changeSaveButton( bool modified ) {
  
  saveButton->setDisabled( !modified );
  
}

void KSpellEdit::changeFile( int id ) {
 
  if (fileSelect->currentItem() != 0) {
     if (!editBox->openFile( spellPath->text() + "/" + fileSelect->currentText() ) )
      qDebug("Open failed for:" + spellPath->text() + "/" + fileSelect->currentText() );
  }
 
}
