/***************************************************************************
                          kspelllist.cpp  -  description
                             -------------------
    begin                : Sun May 12 2002
    copyright            : (C) 2002 by Zach Hobbs
    email                : zach@firstwavedesign.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kspelllist.h"

KSpellList::KSpellList( QWidget* parent=0, const char *name=0 )
   : KListView(parent, name) {
  
  setRootIsDecorated( TRUE ); 
  
  addColumn( "Spell" );
  addColumn( "Version" );
  addColumn( "Latest" );
  addColumn( "Status" ); 
  setColumnWidthMode( 1, KListView::Manual );
  setColumnWidthMode( 2, KListView::Manual );
  setColumnWidthMode( 3, KListView::Manual );
  setColumnWidth( 0, 187);
  setColumnWidth( 1, 90 );
  setColumnWidth( 2, 90 );
  setColumnWidth( 3, 50 );

  //Popup Menu
  QObject::connect( this, SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int) ),
		    this, SLOT(popUpMenu( QListViewItem *, const QPoint &, int ) ) );

  //Do we need to build a grimoire?
  QObject::connect( this, SIGNAL(expanded(QListViewItem *)),
		    this, SLOT(itemExpanded(QListViewItem *)));


}

KSpellList::~KSpellList(){
}

//
void KSpellList::buildList() {

  QString dir;
  QString grimoireName;
  int i;

  i = 0;
  
  while ( i != -1 ) {

    dir = codexDir( i );

    if (dir == "0")
      i = -1;
    else {
      grimoireName = dir.right( dir.length() - dir.findRev("/") - 1 );

      KListViewItem * tmp = new KListViewItem( this, 
					       "[" + QString::number(i) + "] " 
					       + grimoireName, 
					       "", "", "", dir );
      (void) new KListViewItem( tmp, "Empty", "", "", "", "empty" );
      i++;
    }      
    
  }

}

// Gets all the files and creates in the list of files
void KSpellList::buildGrimoire( QListViewItem * item ){
  
  // Create QDir for grimoire and sections dir
  QDir grimoire( item->text(4) );


  if (!grimoire.exists())
    return;

  //QDir grimoire( "/var/lib/sorcery/codex/test");
  
  //Only show me the dir in there
  grimoire.setFilter( QDir::Dirs );
  
  //Creating const list of the Sections
  const QFileInfoList* sectionlist = grimoire.entryInfoList();
    
  // Iterator to go through above list
  QFileInfoListIterator it( *sectionlist );
  
  //Qfile info for the file
  QFileInfo* fi;
  
  //Iterate up 2, skipping "." and ".."
  it += 2;

  // Create progress dialog
  QProgressDialog spellprogress( "Building SpellList", "Quit",
				 grimoire.count() - 2,this, "spellprogress", 
				 TRUE );
  QObject::connect( &spellprogress, SIGNAL(cancelled()),
		    kapp, SLOT(quit()));
  
  int i = 0;

  while ( (fi = it.current()) ) {

    spellprogress.setProgress( ++i );

    KListViewItem * tmp = new KListViewItem( item, fi->fileName(), "", "", "", fi->absFilePath() );
    
    buildSpells( tmp );
    
    ++it;
    
   }

}

void KSpellList::buildSpells( KListViewItem * item ){
   
 // Regular Expression object for regexes
  QRegExp exp;
  
  //Strings for table
  QString myver, status, latestver;
  
  //File that points to the DETAILS
  QFile detailsfile;
  
  //QByteArray for the details file
  QByteArray details;
  
  // QByteArray to store the contents of the installed packages
  QByteArray state;
  
  //OLD - and above was static -- removed because of refresh feature
  //    - could be usefull if it takes to long to build list.
        //Tests to see if function has been called earlier
        //So it doesn't have to open the file for every package
        //if ( state.isNull() ) {
  QFile stateFile( "/var/state/sorcery/packages" );
  stateFile.open( IO_ReadOnly );
  state = stateFile.readAll();
  stateFile.close();
         //}

  // QDir that will be the dir of the section	
  QDir section( item->text(4) );

  if (!section.exists())
    return;

  section.setFilter( QDir::Dirs );

  //Creating const list of the Sections
  const QFileInfoList* spelllist = section.entryInfoList();
  
  // Iterator to go through above list
  QFileInfoListIterator it( *spelllist );
  
  //Qfile info for the file that the iterator is on
  QFileInfo* fi;
  
  //Iterate up 2, skipping "." and ".."
  it += 2;

  // Loop through the file list until no more files   
  
  while ( ( fi = it.current() ) ) {
     
    detailsfile.setName( fi->absFilePath() + "/DETAILS");
    if (!detailsfile.exists()) {
      qDebug( "File Not found: " + fi->absFilePath() + "/DETAILS" );
    } else {

      detailsfile.open( IO_ReadOnly );
      details = detailsfile.readAll();
      detailsfile.close();

      exp.setPattern( "[^#]\\s*VERSION=([^\\n]+)\\n" );

      if ( exp.search(details) > -1 )
      {
	latestver = exp.cap(1);
	latestver = latestver.stripWhiteSpace();
	//	if (latestver.contains( "$" ) )
	//	  qDebug("Version contains $: " + fi->absFilePath() );
      }
      else
	qDebug("No Version " + fi->absFilePath() );

      // Regexp to find myversion and status

      exp.setPattern( "\\n" + QRegExp::escape(fi->fileName()) + ":([0-9]+):([^:]+):([^:]+)\\n" );

      if (exp.search( state ) > -1) {
	status = exp.cap(2);
	myver = exp.cap(3);
	
	status = status.stripWhiteSpace();
	myver = myver.stripWhiteSpace();
	
	if (myver.length() == 0)
	  myver = "-";
      } 
      else {
	status = "-";
	myver = "-";
      }


      if ( myver == latestver )
	(void) new KSpellListItem( item, QColor( 0,150,0 ), fi->fileName(), myver, latestver, status, fi->absFilePath() ); 
      else if ( myver != "-" )
	(void) new KSpellListItem( item, QColor( 255, 0, 0 ), fi->fileName(), myver, latestver, status, fi->absFilePath() );
      else
	(void) new KSpellListItem( item, QColor( 0, 0, 0 ), fi->fileName(), myver, latestver, status, fi->absFilePath() ); 
    }
    ++it;
  }
}

QString KSpellList::codexDir( int id ) {
  
  QByteArray codexesArray;
  QFile codexes("/etc/sorcery/local/grimoire");
  QRegExp exp;

  if (!codexes.exists() )
    return "0";

  codexes.open( IO_ReadOnly );
  codexesArray = codexes.readAll();
  codexes.close();
  
  exp.setPattern("GRIMOIRE_DIR\\[" + QString::number(id) + "\\]=([^\\n]+)\\n");

  if ( exp.search( codexesArray ) > -1 ) {

    return exp.cap(1);

  } else {
    return "0";
  }
}


void KSpellList::refreshList( void ) {

  QListViewItem *tmp = this->currentItem();
  
  while ( tmp->depth() != 0 )
    tmp = tmp->parent();

  while (tmp->firstChild()) 
    delete tmp->firstChild();
  
  buildGrimoire( tmp );

}

void KSpellList::popUpMenu( QListViewItem *spell, const QPoint & point, 
			    int level ) {
  
  if (!spell)
    return;

  contextMenu = new QPopupMenu( this );
  
  contextMenu->insertItem( "Refresh List", this, SLOT( refreshList() ),0,0 );


  if (spell->depth() == 2) {
    contextMenu->insertSeparator();

    if ( spell->text(3) == "installed" || spell->text(3) == "held" ) {
      contextMenu->insertItem( "Dispel Spell" );      
      contextMenu->insertItem( "Cast -r -c Spell" );

      if ( spell->text(2) != spell->text(1) ) {
	contextMenu->insertItem( "Summon Spell" );
      }
      
    } else {
      contextMenu->insertItem( "Add to Queue", 1 );
      contextMenu->insertItem( "Cast Spell" );
      contextMenu->insertItem( "Summon Spell" );
    }
    QObject::connect( contextMenu, SIGNAL(activated ( int ) ),
		      this, SLOT(popUpAction( int ) ) );
  }
  
  contextMenu->exec( point );
}

void KSpellList::popUpAction( int id ) {

  if (id == 1) {
    QString *spellname = new QString(this->currentItem()->text(0));
    emit addQueue( spellname );

  } else if (id != 0) {
    QString* cmd;

    cmd = new QString( contextMenu->text( id ) );
    *cmd = cmd->lower();
    *cmd->remove( "spell" );
    *cmd += this->currentItem()->text(0);
  
    emit actionTaken( cmd );
  }
}

void KSpellList::itemExpanded( QListViewItem * item ) {

  if (item->depth() == 0) {
    if( item->firstChild()->text(4) == "empty" ) {
      QString dir;
      QString grimoireName;
      int question;

      dir = item->text(4);
      grimoireName = dir.right( dir.length() - dir.findRev("/") - 1 );
      
      question = KMessageBox::questionYesNo( this,
					     "Codex \"" + grimoireName + "\" is\n" +
					     "currently empty.  Would you like to\n" +
					     "build it now?\n",
					     "Ksorcery");
					     
      if (question == 3) {
	item->takeItem( item->firstChild() );
	buildGrimoire( item );
      }
      else
	setOpen( item, false );
    }
  }

}
